#include "blc_text.h"
#include "blc_channel.h"
#include "blc_program.h"
#include <fftw3.h>
#include <unistd.h>
#include <math.h> //sqrtf
#include <pthread.h>
#include <memory>

using namespace std;

#define DEFAULT_OUTPUT_NAME "spectrum<pid>"

sem_t *sem_output, *sem_input;
int i=0;
struct timeval;
char const *display, *option_record;
int columns_nb=64, rows_nb=16;

int main(int argc, char**argv){
    blc_channel input;
    unique_ptr<blc_channel> output;
    char const *input_name, *output_name, *spectrum_option, *period_str;
    FILE *file=nullptr;
    int ret;
    int output_length, period, pos=0;
    size_t i;
    fftwf_complex *intermediate;
    fftwf_plan fftw_plan;
    
    blc_program_add_option(&display, 'd', "display", nullptr, "Display the result as text graph", nullptr);
    blc_program_add_option(&output_name, 'o', "output", "blc_channel|-", "channel containing the fast fourier transformation, or '-' for terminal output", DEFAULT_OUTPUT_NAME);
    blc_program_add_option(&period_str, 'p', "period", "integer", "Period in ms (0 as fast as possible).", "0");
    blc_program_add_option(&spectrum_option, 's', "spectrum", nullptr, "Get the square value of the fftw signal", nullptr);
    blc_program_add_parameter(&input_name, "input blc_channel", 1, "Channel containing the sound data", nullptr);
    blc_program_init(&argc, &argv, blc_quit);
    SSCANF(1, period_str, "%d", &period);

    //Open input channel and check the type of values
    input.open(input_name, BLC_CHANNEL_READ);
    if (input.type!='FL32') EXIT_ON_CHANNEL_ERROR(&input,"Input must be of type 'FL32'");
    output_length=input.total_length/2+1;
    vector<size_t> dims = { static_cast<size_t>(output_length) };


    if (strcmp(output_name, DEFAULT_OUTPUT_NAME)==0) {
        asprintf((char**)&output_name, ":spectrum%d", getpid());
        output=make_unique<blc_channel>(output_name, BLC_CHANNEL_WRITE, 'FL32', 'NDEF', dims);
        output->publish();

        blc_loop_try_add_waiting_semaphore(output->sem_ack_data);
        blc_loop_try_add_posting_semaphore(output->sem_new_data);
    }
    else{
        ret=sscanf(output_name, "%*[:/.^]%*[^/]%n", &pos);
        if (pos==strlen(output_name)){
            output = make_unique<blc_channel> (output_name, BLC_CHANNEL_WRITE, 'FL32', 'NDEF',  dims);
            output->publish();

            blc_loop_try_add_waiting_semaphore(output->sem_ack_data);
            blc_loop_try_add_posting_semaphore(output->sem_new_data);
        }
        else //C'est un format de fichier
        {

            EXIT_ON_ERROR("File not managed");
//            if (strcmp(blc_get_filename_extension(output_name), "tsv")!=0) EXIT_ON_ERROR("Filename extension must be .tsv but your file is: '%s'.\nThis is to be sure you are not overwriting important file by mistake", output_name);
//             output=make_unique<blc_channel>('FL32', 'NDEF', dims);
//            SYSTEM_ERROR_CHECK(file=fopen(output_name,"w"), nullptr, "opening '%s'", output_name);
        }
    }

    blc_loop_try_add_waiting_semaphore(input.sem_new_data);
    blc_loop_try_add_posting_semaphore(input.sem_ack_data);
    
    intermediate=fftwf_alloc_complex(output_length);
    fftw_plan = fftwf_plan_dft_r2c_1d(input.total_length, input.floats, intermediate , FFTW_ESTIMATE);

    BLC_COMMAND_LOOP(period*1000){
        fftwf_execute(fftw_plan); /* repeat as needed */
        FOR(i, output_length){
            output->floats[i]=sqrtf(intermediate[i][0]*intermediate[i][0]+intermediate[i][1]*intermediate[i][1])/input.total_length;
        }
        //if output is not a channel we export to standard output
        if (file) {
            // if (blc_loop_iteration && file==stdout && blc_output_terminal) blc_eprint_cursor_up(100);
            output->fprint_tsv(file);
        }
        if(display){
            blc_fprint_float_graph(stderr, output->floats, output->total_length, "spectrum", columns_nb, rows_nb, 1, 0, "Frequency", "Intensity");
            blc_eprint_cursor_up(rows_nb);
        }
    }

    fftwf_destroy_plan(fftw_plan);
    blc_eprint_cursor_down(rows_nb);
    
    if (file) SYSTEM_ERROR_CHECK(fclose(file), -1, nullptr);
    return EXIT_SUCCESS;
}

